@@ -9,6 +9,7 @@ import tempfile
 
 import requests
 import yaml
+from path import Path as path
 
 
 log = logging.getLogger(__name__)
@@ -107,8 +108,6 @@ class Fetcher(object):
         return match.groupdict() if match else {}
 
     def get_revision(self, dir_):
-        if self.revision:
-            return self.revision
         for cmd in ("git rev-parse HEAD",
                     "bzr revision-info",
                     "hg log -l 1 --template '{node}\n' -r ."):
@@ -122,6 +121,25 @@ class Fetcher(object):
                     return rev_info.decode('utf8').strip()
             except FetchError:
                 continue
+        rev_file = path(dir_) / '.pull-source-rev'
+        if rev_file.exists():
+            return rev_file.read_text().strip()
+        return self.revision
+
+    def get_branch_for_revision(self, dir_, revision=None):
+        """Returns None if the revision doesn't match the actual branch name"""
+        if revision is None:
+            revision = self.revision
+        if not revision:
+            return None
+        for cmd in ("git branch --contains {} --format=\"%(refname)\""
+                    .format(revision), ):
+            try:
+                branch = check_output(cmd, cwd=dir_).decode('UTF-8').strip()
+                return branch
+            except FetchError as e:
+                log.debug("get_branch_for_revision: got error: ", e)
+                continue
         return None
 
 
@@ -139,8 +157,11 @@ class BzrFetcher(Fetcher):
     def fetch(self, dir_):
         dir_ = tempfile.mkdtemp(dir=dir_)
         url = 'lp:' + self.repo
+        self.fetched_url = url
+        self.vcs = "bzr"
         cmd = 'branch --use-existing-dir {} {}'.format(url, dir_)
         if self.revision:
+            log.debug('Switching to revision: {}'.format(self.revision))
             cmd = '{} -r {}'.format(cmd, self.revision)
         bzr(cmd)
         return rename(dir_)
@@ -156,6 +177,8 @@ class BzrMergeProposalFetcher(BzrFetcher):
         dir_ = tempfile.mkdtemp(dir=dir_)
         api_base = 'https://api.launchpad.net/devel/'
         url = api_base + self.repo
+        self.fetched_url = url
+        self.vcs = "bzr"
         merge_data = get(url).json()
         target = 'lp:' + merge_data['target_branch_link'][len(api_base):]
         source = 'lp:' + merge_data['source_branch_link'][len(api_base):]
@@ -174,8 +197,11 @@ class LaunchpadGitFetcher(Fetcher):
     def fetch(self, dir_):
         dir_ = tempfile.mkdtemp(dir=dir_)
         url = 'https://git.launchpad.net/' + self.repo
+        self.fetched_url = url
+        self.vcs = "git"
         git('clone {} {}'.format(url, dir_))
         if self.revision:
+            log.debug('Switching to revision: {}'.format(self.revision))
             git('checkout {}'.format(self.revision), cwd=dir_)
         return rename(dir_)
 
@@ -189,8 +215,11 @@ class GithubFetcher(Fetcher):
     def fetch(self, dir_):
         dir_ = tempfile.mkdtemp(dir=dir_)
         url = 'https://github.com/' + self.repo
+        self.fetched_url = url
+        self.vcs = "git"
         git('clone {} {}'.format(url, dir_))
         if self.revision:
+            log.debug('Switching to revision: {}'.format(self.revision))
             git('checkout {}'.format(self.revision), cwd=dir_)
         return rename(dir_)
 
@@ -204,8 +233,11 @@ class OpendevFetcher(Fetcher):
     def fetch(self, dir_):
         dir_ = tempfile.mkdtemp(dir=dir_)
         url = 'https://opendev.org/' + self.repo
+        self.fetched_url = url
+        self.vcs = "git"
         git('clone {} {}'.format(url, dir_))
         if self.revision:
+            log.debug('Switching to revision: {}'.format(self.revision))
             git('checkout {}'.format(self.revision), cwd=dir_)
         return rename(dir_)
 
@@ -222,7 +254,12 @@ class GitFetcher(Fetcher):
 
     def fetch(self, dir_):
         dir_ = tempfile.mkdtemp(dir=dir_)
+        self.fetched_url = self.repo
+        self.vcs = "git"
         git('clone {} {}'.format(self.repo, dir_))
+        if self.revision:
+            log.debug('Switching to revision: {}'.format(self.revision))
+            git('checkout {}'.format(self.revision), cwd=dir_)
         return rename(dir_)
 
 
@@ -235,19 +272,24 @@ class BitbucketFetcher(Fetcher):
     def fetch(self, dir_):
         dir_ = tempfile.mkdtemp(dir=dir_)
         url = 'https://bitbucket.org/' + self.repo
+        self.fetched_url = url
         if url.endswith('.git'):
+            self.vcs = "git"
             return self._fetch_git(url, dir_)
+        self.vcs = "hg"
         return self._fetch_hg(url, dir_)
 
     def _fetch_git(self, url, dir_):
         git('clone {} {}'.format(url, dir_))
         if self.revision:
+            log.debug('Switching to revision: {}'.format(self.revision))
             git('checkout {}'.format(self.revision), cwd=dir_)
         return rename(dir_)
 
     def _fetch_hg(self, url, dir_):
         cmd = 'clone {} {}'.format(url, dir_)
         if self.revision:
+            log.debug('Switching to revision: {}'.format(self.revision))
             cmd = '{} -u {}'.format(cmd, self.revision)
         hg(cmd)
         return rename(dir_)
@@ -285,6 +327,8 @@ class CharmstoreDownloader(Fetcher):
 
     def fetch(self, dir_):
         url = self.ARCHIVE_URL.format(self.entity)
+        self.fetched_url = url
+        self.vcs = "charmstore"
         archive = download_file(url, dir_)
         entity_dir = extract_archive(archive, dir_)
         return rename(entity_dir)
